#ifndef GLABELBACKUPINFO_H
#define GLABELBACKUPINFO_H

#include "common/singleton.h"
#include "common/spinlock_mutex.h"
#include "globalsignals.h"
#include "logic/KMBuilderManifest.h"

class GlobelInfo : public Singleton<GlobelInfo> {
public:
    enum BlackgroundMode {
        Auto,
        Light,
        Dark
    };

    GlobelInfo(token) {}
    ~GlobelInfo() {}

    void setMainWidget(QWidget * mainWidget) {
        m_mainWidget = mainWidget;
    }

    QWidget * getMainWidget() {
        return m_mainWidget;
    }

    KMBuilderManifest* getManifest() {
        return &m_manifest;
    }

    // 设置全局项目目录
    void setProjectDir(const QString &dir) {
        projectDir = dir;
    }

    // 获取全局项目目录
    QString getProjectDir() const {
        return projectDir;
    }

    // 设置主题背景模式
    void setBlackgroundMode(BlackgroundMode blackgroundMode) {
        m_BlackgroundMode = blackgroundMode;
    }

    // 主题背景模式
    BlackgroundMode getBlackgroundMode() {
        return m_BlackgroundMode;
    }

    GlobalSignals * getGlobalSignals() {
        return &m_globalSignals;
    }

    void lock() {
        m_outerMutex.lock();
    }

    void unlock() {
        m_outerMutex.unlock();
    }

    bool try_lock() {
        return m_outerMutex.try_lock();
    }

private:
    // 是否跟随主题
    BlackgroundMode m_BlackgroundMode = BlackgroundMode::Auto;

    QWidget * m_mainWidget = nullptr;
    KMBuilderManifest m_manifest;
    QString projectDir;

    // 全局信号对象
    GlobalSignals m_globalSignals;

    spinlock_mutex m_interMutex;
    spinlock_mutex m_outerMutex;
};

#endif // GLABELBACKUPINFO_H
