#include "KMYAMLNodeHelper.h"

YAML::Node KMYAMLNodeHelper::loadFromYAML(const QString& yamlFile)
{
    YAML::Node rootNode = YAML::LoadFile(yamlFile.toStdString());
    return rootNode;
}

QString KMYAMLNodeHelper::getStringValue(const QString &key, const QString &defaultValue)
{
    YAML::Node value = m_node[key.toStdString()];
    if (!value.IsDefined())
    {
        return defaultValue;
    }

    if (value.IsScalar())
    {
        return QString::fromStdString(value.as<std::string>());
    }

    return defaultValue;
}

long KMYAMLNodeHelper::getLongValue(const QString &key, long defaultValue)
{
    YAML::Node value = m_node[key.toStdString()];
    if (!value.IsDefined())
    {
        return defaultValue;
    }

    if (value.IsScalar())
    {
        return value.as<long>();
    }

    return defaultValue;
}

int KMYAMLNodeHelper::getIntValue(const QString &key, int defaultValue)
{
    YAML::Node value = m_node[key.toStdString()];
    if (!value.IsDefined())
    {
        return defaultValue;
    }

    if (value.IsScalar())
    {
        return value.as<int>();
    }

    return defaultValue;
}

bool KMYAMLNodeHelper::getBoolValue(const QString &key, bool defaultValue)
{
    YAML::Node value = m_node[key.toStdString()];
    if (!value.IsDefined())
    {
        return defaultValue;
    }

    if (value.IsScalar())
    {
        return value.as<bool>();
    }

    return defaultValue;
}

QVector<QString> KMYAMLNodeHelper::getStringListValue(const QString &key)
{
    QVector<QString> defaultValue;

    YAML::Node value = m_node[key.toStdString()];
    if (!value.IsDefined())
    {
        return defaultValue;
    }

    if (value.IsSequence())
    {
        for (YAML::Node::const_iterator iter = value.begin(); iter != value.end(); iter++)
        {
            if ((*iter).IsScalar())
            {
                defaultValue.push_back(QString::fromStdString((*iter).as<std::string>()));
            }
        }
    }

    return defaultValue;
}

QStringList KMYAMLNodeHelper::getStringListValue1(const QString &key) const
{
    QStringList result;

    YAML::Node value = m_node[key.toStdString()];
    if (!value.IsDefined() || !value.IsSequence())
    {
        return result; // 返回空列表
    }

    for (const auto &item : value)
    {
        if (item.IsScalar())
        {
            result.append(QString::fromStdString(item.as<std::string>()));
        }
    }

    return result;
}

QMap<QString, QString> KMYAMLNodeHelper::getStringMapValue(const QString &key)
{
    QMap<QString, QString> defaultValue;

    YAML::Node node = m_node[key.toStdString()];
    if (!node.IsDefined())
    {
        return defaultValue;
    }

    if (node.IsMap())
    {
        for(YAML::Node::const_iterator iter = node.begin() ; iter != node.end() ; iter++)
        {
            QString key = QString::fromStdString(iter->first.as<std::string>());
            YAML::Node value = iter->second;
            if (value.IsScalar())
            {
                defaultValue[key] = QString::fromStdString(value.as<std::string>());
            }
        }
    }
    return defaultValue;
}
