#include "permissiondialog.h"
#include <QDebug>

PermissionDialog::PermissionDialog(QWidget *parent)
    : QDialog(parent)
{
    setWindowTitle("权限管理");
    initUI();
}

void PermissionDialog::initUI()
{
    QVBoxLayout *mainLayout = new QVBoxLayout(this);

    // 文件系统权限
    QGroupBox *filesystemGroupBox = new QGroupBox(tr("filesystem"), this);
    QGridLayout *filesystemLayout = new QGridLayout;
    m_xdgUserDirsCheckBox = new QCheckBox(tr("xdg-user-dirs"), this);
    filesystemLayout->addWidget(m_xdgUserDirsCheckBox, 0, 0);

    QLabel *filesystemDirsLabel = new QLabel(tr("filesystem-dirs"), this);
    filesystemLayout->addWidget(filesystemDirsLabel, 1, 0);
    m_filesystemDirsLineEdit = new QTextEdit(this);
    m_filesystemDirsLineEdit->setPlaceholderText(tr("User-specified directory paths. "
                                    "Example: "
                                    "/path/a, /path/b"));
    m_filesystemDirsLineEdit->setToolTip(tr("User-specified directory paths, Separated with commas.\n"
                            "Example:\n"
                            "/path/a, /path/b"));
    filesystemLayout->addWidget(m_filesystemDirsLineEdit, 1, 1);
    filesystemGroupBox->setLayout(filesystemLayout);
    mainLayout->addWidget(filesystemGroupBox);

    // 网络权限
    QGroupBox *m_netGroupBox = new QGroupBox(tr("net"), this);
    QGridLayout *netLayout = new QGridLayout;
    m_networkCheckBox = new QCheckBox(tr("network"), this);
    netLayout->addWidget(m_networkCheckBox, 0, 0);

    m_scanningCheckBox = new QCheckBox(tr("scanning"), this);
    netLayout->addWidget(m_scanningCheckBox, 0, 1);
    m_netGroupBox->setLayout(netLayout);
    mainLayout->addWidget(m_netGroupBox);

    // 设备权限
    QGroupBox *deviceGroupBox = new QGroupBox(tr("device"), this);
    QGridLayout *deviceLayout = new QGridLayout;
    m_bluetoothCheckBox = new QCheckBox(tr("bluetooth"), this);
    deviceLayout->addWidget(m_bluetoothCheckBox, 0, 0);

    m_microphoneCheckBox = new QCheckBox(tr("microphone"), this);
    deviceLayout->addWidget(m_microphoneCheckBox, 0, 1);

    m_cameraCheckBox = new QCheckBox(tr("camera"), this);
    deviceLayout->addWidget(m_cameraCheckBox, 1, 0);

    m_usbCheckBox = new QCheckBox(tr("usb"), this);
    deviceLayout->addWidget(m_usbCheckBox, 1, 1);

    m_serialCheckBox = new QCheckBox(tr("serial"), this);
    deviceLayout->addWidget(m_serialCheckBox, 2, 0);
    deviceGroupBox->setLayout(deviceLayout);
    mainLayout->addWidget(deviceGroupBox);

    // 进程管理
    QGroupBox *m_processGroupBox = new QGroupBox(tr("processManager"), this);
    QGridLayout *processLayout = new QGridLayout;
    m_autostartCheckBox = new QCheckBox(tr("autostart"), this);
    processLayout->addWidget(m_autostartCheckBox, 0, 0);
    m_processGroupBox->setLayout(processLayout);
    mainLayout->addWidget(m_processGroupBox);

    // 其他权限
    QGroupBox *othersGroupBox = new QGroupBox(tr("others"), this);
    QGridLayout *othersLayout = new QGridLayout;
    m_systemNoticeCheckBox = new QCheckBox(tr("system-notice"), this);
    othersLayout->addWidget(m_systemNoticeCheckBox, 0, 0);

    m_systemTimeCheckBox = new QCheckBox(tr("system-time"), this);
    othersLayout->addWidget(m_systemTimeCheckBox, 0, 1);

    m_systemServerCheckBox = new QCheckBox(tr("system-server"), this);
    othersLayout->addWidget(m_systemServerCheckBox, 1, 0);

    m_windowTopCheckBox = new QCheckBox(tr("window-top"), this);
    othersLayout->addWidget(m_windowTopCheckBox, 1, 1);

    m_preventSleepCheckBox = new QCheckBox(tr("prevent-sleep"), this);
    othersLayout->addWidget(m_preventSleepCheckBox, 2, 0);

    m_systemDisplayResolutionCheckBox = new QCheckBox(tr("system-display-resolution"), this);
    othersLayout->addWidget(m_systemDisplayResolutionCheckBox, 2, 1);

    m_systemDisplayBrightnessCheckBox = new QCheckBox(tr("system-display-brightness"), this);
    othersLayout->addWidget(m_systemDisplayBrightnessCheckBox, 3, 0);

    m_systemAudioSettingCheckBox = new QCheckBox(tr("system-audio-setting"), this);
    othersLayout->addWidget(m_systemAudioSettingCheckBox, 3, 1);

    m_systemAudioSizeCheckBox = new QCheckBox(tr("system-audio-size"), this);
    othersLayout->addWidget(m_systemAudioSizeCheckBox, 4, 0);

    m_systemCrontabCheckBox = new QCheckBox(tr("system-crontab"), this);
    othersLayout->addWidget(m_systemCrontabCheckBox, 4, 1);

    m_systemPowerModeCheckBox = new QCheckBox(tr("system-power-mode"), this);
    othersLayout->addWidget(m_systemPowerModeCheckBox, 5, 0);

    m_systemPowerPolicyCheckBox = new QCheckBox(tr("system-power-policy"), this);
    othersLayout->addWidget(m_systemPowerPolicyCheckBox, 5, 1);

    othersGroupBox->setLayout(othersLayout);
    mainLayout->addWidget(othersGroupBox);

    // 确定按钮
    QPushButton *okButton = new QPushButton(tr("Confirm"), this);
    connect(okButton, &QPushButton::clicked, this, [=](){
         QDialog::accept();
    });
    mainLayout->addWidget(okButton);

    setLayout(mainLayout);
}

void PermissionDialog::setPermissionData(const KMBuilderManifest::Permission &permission)
{
    // 加载文件系统权限
    m_xdgUserDirsCheckBox->setChecked(permission.m_filesystem.m_xdgUserDirs);
    m_filesystemDirsLineEdit->setText(permission.m_filesystem.m_filesystemDirs.join(","));

    // 加载网络权限
    m_networkCheckBox->setChecked(permission.m_net.m_network);
    m_scanningCheckBox->setChecked(permission.m_net.m_scanning);

    // 加载设备权限
    m_bluetoothCheckBox->setChecked(permission.m_device.m_bluetooth);
    m_microphoneCheckBox->setChecked(permission.m_device.m_microphone);
    m_cameraCheckBox->setChecked(permission.m_device.m_camera);
    m_usbCheckBox->setChecked(permission.m_device.m_usb);
    m_serialCheckBox->setChecked(permission.m_device.m_serial);

    //加载进程管理
    m_autostartCheckBox->setChecked(permission.m_processManager.m_autostart);

    // 加载其他权限
    m_systemNoticeCheckBox->setChecked(permission.m_others.m_systemNotice);
    m_systemTimeCheckBox->setChecked(permission.m_others.m_systemTime);
    m_systemServerCheckBox->setChecked(permission.m_others.m_systemServer);
    m_windowTopCheckBox->setChecked(permission.m_others.m_windowTop);
    m_preventSleepCheckBox->setChecked(permission.m_others.m_preventSleep);
    m_systemDisplayResolutionCheckBox->setChecked(permission.m_others.m_systemDisplayResolution);
    m_systemDisplayBrightnessCheckBox->setChecked(permission.m_others.m_systemDisplayBrightness);
    m_systemAudioSettingCheckBox->setChecked(permission.m_others.m_systemAudioSetting);
    m_systemAudioSizeCheckBox->setChecked(permission.m_others.m_systemAudioSize);
    m_systemCrontabCheckBox->setChecked(permission.m_others.m_systemCrontab);
    m_systemPowerModeCheckBox->setChecked(permission.m_others.m_systemPowerMode);
    m_systemPowerPolicyCheckBox->setChecked(permission.m_others.m_systemPowerPolicy);
}

KMBuilderManifest::Permission PermissionDialog::getPermissionData() const
{
   KMBuilderManifest::Permission permission;
   // 文件系统权限
   permission.m_filesystem.m_xdgUserDirs = m_xdgUserDirsCheckBox->isChecked();
#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
    permission.m_filesystem.m_filesystemDirs = m_filesystemDirsLineEdit->toPlainText().split(",", Qt::SkipEmptyParts);
#else
    permission.m_filesystem.m_filesystemDirs = m_filesystemDirsLineEdit->toPlainText().split(",", QString::SkipEmptyParts);
#endif

   // 网络权限
   permission.m_net.m_network = m_networkCheckBox->isChecked();
   permission.m_net.m_scanning = m_scanningCheckBox->isChecked();

   // 设备权限
   permission.m_device.m_bluetooth = m_bluetoothCheckBox->isChecked();
   permission.m_device.m_microphone = m_microphoneCheckBox->isChecked();
   permission.m_device.m_camera = m_cameraCheckBox->isChecked();
   permission.m_device.m_usb = m_usbCheckBox->isChecked();
   permission.m_device.m_serial = m_serialCheckBox->isChecked();

   // 进程管理
   permission.m_processManager.m_autostart = m_autostartCheckBox->isChecked();

   // 其他权限
   permission.m_others.m_systemNotice = m_systemNoticeCheckBox->isChecked();
   permission.m_others.m_systemTime = m_systemTimeCheckBox->isChecked();
   permission.m_others.m_systemServer = m_systemServerCheckBox->isChecked();
   permission.m_others.m_windowTop = m_windowTopCheckBox->isChecked();
   permission.m_others.m_preventSleep = m_preventSleepCheckBox->isChecked();
   permission.m_others.m_systemDisplayResolution = m_systemDisplayResolutionCheckBox->isChecked();
   permission.m_others.m_systemDisplayBrightness = m_systemDisplayBrightnessCheckBox->isChecked();
   permission.m_others.m_systemAudioSetting = m_systemAudioSettingCheckBox->isChecked();
   permission.m_others.m_systemAudioSize = m_systemAudioSizeCheckBox->isChecked();
   permission.m_others.m_systemCrontab = m_systemCrontabCheckBox->isChecked();
   permission.m_others.m_systemPowerMode = m_systemPowerModeCheckBox->isChecked();
   permission.m_others.m_systemPowerPolicy = m_systemPowerPolicyCheckBox->isChecked();

   return permission ;
}

